@@ -1,24 +1,30 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
- *
- * This code is licensed under the terms of the GPLv2
- * <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
- * MySQL Connectors. There are special exceptions to the terms and
- * conditions of the GPLv2 as it is applied to this software, see the
- * FLOSS License Exception
- * <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+ * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; version 2 of the License.
+ * it under the terms of the GNU General Public License, version 2.0, as
+ * published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an
+ * additional permission to link the program and your derivative works
+ * with the separately licensed software that they have included with
+ * MySQL.
+ *
+ * Without limiting anything contained in the foregoing, this file,
+ * which is part of MySQL Connector/C++, is also subject to the
+ * Universal FOSS Exception, version 1.0, a copy of which can be found at
+ * http://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License, version 2.0, for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
@@ -110,11 +116,17 @@ protected:
       }
   */
 
-  enum { START, MDATA, ROWS, CLOSE, DONE } m_result_state, m_next_state;
+  enum result_state_t
+  { START, MDATA, ROWS, CLOSE, DONE };
+
+  result_state_t m_result_state =START, m_next_state;
   row_count_t m_rcount;
   col_count_t m_ccount;
 
-  bool is_done() const { return DONE == m_result_state; }
+  bool is_done() const
+  {
+    return DONE == m_result_state;
+  }
 
   /*
     These functions determine how to process each incoming message and
@@ -414,10 +426,10 @@ Op_rcv::Next_msg Rcv_result_base::do_next_msg(msg_type_t type)
       the following messages:
 
       - Row - next row from the row-set, we continue reading rows until we
-              see <more>;
+        see <more>;
 
       - FetchDoneXXX - these messages start <more> sequence; they are consumed
-                       as part of this stage and the next stage starts.
+        as part of this stage and the next stage starts.
     */
 
     switch (type)
@@ -428,6 +440,9 @@ Op_rcv::Next_msg Rcv_result_base::do_next_msg(msg_type_t type)
       break;
     case msg_type::FetchDoneMoreResultsets:
       m_next_state = MDATA;  // proceed to next result-set
+
+
+
       break;
     default: return UNEXPECTED;
     };
@@ -522,8 +537,9 @@ namespace mysqlx {
 
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Resultset::FetchDoneMoreResultsets &msg,
-                                       Row_processor &rp)
+void Rcv_result_base::process_msg_with(
+  Mysqlx::Resultset::FetchDoneMoreResultsets&, Row_processor &rp
+)
 {
   /*
     Inform the processor about finishing reading all rows from the
@@ -536,8 +552,9 @@ void Rcv_result_base::process_msg_with(Mysqlx::Resultset::FetchDoneMoreResultset
 
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Resultset::FetchDone &msg,
-                                       Row_processor &rp)
+void Rcv_result_base::process_msg_with(
+  Mysqlx::Resultset::FetchDone&, Row_processor &rp
+)
 {
   /*
     Fetching all rows from the cursor is finished.
@@ -550,8 +567,9 @@ void Rcv_result_base::process_msg_with(Mysqlx::Resultset::FetchDone &msg,
 
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Resultset::Row &row,
-                                       Row_processor &rp)
+void Rcv_result_base::process_msg_with(
+  Mysqlx::Resultset::Row &row, Row_processor &rp
+)
 {
   row_count_t rcount= m_rcount++;
 
@@ -594,8 +612,9 @@ void Rcv_result_base::process_msg_with(Mysqlx::Resultset::Row &row,
 
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Resultset::ColumnMetaData &col_mdata,
-                                       Mdata_processor &mdata_proc)
+void Rcv_result_base::process_msg_with(
+  Mysqlx::Resultset::ColumnMetaData &col_mdata, Mdata_processor &mdata_proc
+)
 {
     col_count_t ccount= m_ccount++;
 
@@ -639,16 +658,16 @@ void Rcv_result_base::process_msg_with(Mysqlx::Resultset::ColumnMetaData &col_md
 }
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Ok &ok,
-                                       Mdata_processor &prc)
+void Rcv_result_base::process_msg_with(Mysqlx::Ok &ok, Mdata_processor &prc)
 {
   prc.ok(ok.msg());
 }
 
 
 template<>
-void Rcv_result_base::process_msg_with(Mysqlx::Sql::StmtExecuteOk &ok,
-                                       Stmt_processor &prc)
+void Rcv_result_base::process_msg_with(
+  Mysqlx::Sql::StmtExecuteOk&, Stmt_processor &prc
+)
 {
   prc.execute_ok();
 }
