@@ -40,6 +40,7 @@
 #include <vector>
 #include <algorithm>
 #include <random>
+#include <mutex>
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -81,7 +82,8 @@
 #else
 #include <string.h>
 #endif
-
+#include "cppconn/callback.h"
+#include "mysql_driver.h"
 #include "mysql_connection.h"
 #include "mysql_connection_data.h"
 #include "mysql_prepared_statement.h"
@@ -95,6 +97,18 @@
 # define ER_MUST_CHANGE_PASSWORD_LOGIN 1820
 #endif
 
+std::mutex callback_mutex;
+
+sql::Fido_Callback * fido_callback_instance = nullptr;
+
+static void fido_callback_func(const char* msg)
+{
+  if (!fido_callback_instance)
+    return;
+  (*fido_callback_instance)(msg);
+}
+
+
 namespace sql
 {
 namespace mysql
@@ -277,6 +291,7 @@ static const String2IntMap intOptions[]=
     {OPT_NET_BUFFER_LENGTH,   MYSQL_OPT_NET_BUFFER_LENGTH, false},
 #endif
     {OPT_SSL_MODE,            MYSQL_OPT_SSL_MODE    , false},
+    {"OPT_SSL_MODE",          MYSQL_OPT_SSL_MODE    , false},
 #if MYCPPCONN_STATIC_MYSQL_VERSION_ID >= 80000
     {OPT_RETRY_COUNT,         MYSQL_OPT_RETRY_COUNT, false},
 #endif
@@ -291,7 +306,9 @@ static const String2IntMap stringOptions[]=
     {OPT_SSL_CAPATH,           MYSQL_OPT_SSL_CAPATH, true},
     {OPT_SSL_CIPHER,           MYSQL_OPT_SSL_CIPHER, true},
     {OPT_SSL_CRL,              MYSQL_OPT_SSL_CRL, false},
+    {"sslCRL",                 MYSQL_OPT_SSL_CRL, false},
     {OPT_SSL_CRLPATH,          MYSQL_OPT_SSL_CRLPATH, false},
+    {"sslCRLPath",             MYSQL_OPT_SSL_CRLPATH, false},
     {OPT_SERVER_PUBLIC_KEY,    MYSQL_SERVER_PUBLIC_KEY, false},
     {OPT_SET_CHARSET_DIR,      MYSQL_SET_CHARSET_DIR, false},
     {OPT_PLUGIN_DIR,           MYSQL_PLUGIN_DIR, false},
@@ -301,7 +318,8 @@ static const String2IntMap stringOptions[]=
     {OPT_READ_DEFAULT_FILE,    MYSQL_READ_DEFAULT_FILE, false},
     {OPT_CHARSET_NAME,         MYSQL_SET_CHARSET_NAME, true},
 #if MYCPPCONN_STATIC_MYSQL_VERSION_ID >= 50700
-    {OPT_TLS_VERSION,          MYSQL_OPT_TLS_VERSION, false},
+    {OPT_TLS_VERSION,          MYSQL_OPT_TLS_VERSION, true},
+    {"OPT_TLS_VERSION",        MYSQL_OPT_TLS_VERSION, true},
 #endif
     {OPT_LOAD_DATA_LOCAL_DIR, MYSQL_OPT_LOAD_DATA_LOCAL_DIR, false}
   };
@@ -486,7 +504,6 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
   MySQL_Uri uri;
   MySQL_Uri::Host_data host;
 
-
   sql::SQLString userName;
   sql::SQLString password;
   sql::SQLString defaultCharset("utf8mb4");
@@ -559,6 +576,7 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
   */
 
   {
+    sql::SQLString plugin_dir;
     it = properties.find(OPT_PLUGIN_DIR);
 
     if (it != properties.end()) {
@@ -567,12 +585,28 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
       } catch (sql::InvalidArgumentException&) {
         throw sql::InvalidArgumentException("Wrong type passed for pluginDir expected sql::SQLString");
       }
-      if (p_s) {
-        proxy->options(sql::mysql::MYSQL_PLUGIN_DIR, *p_s);
-      }
-      else {
-        throw sql::InvalidArgumentException("No string value passed for pluginDir");
-      }
+    }
+#if(_WIN32 && CONCPP_BUILD_SHARED)
+    else {
+      /*
+        Note: For DLL in Windows we will try to set the plugin directory
+        based on driver_dll_path.
+      */
+      plugin_dir = driver_dll_path;
+#ifdef _DEBUG
+      // Debug dll is placed inside debug subdirectory
+      plugin_dir.append("..\\");
+#endif
+      plugin_dir.append("plugin");
+      p_s = &plugin_dir;
+    }
+#endif
+
+    if (p_s) {
+      proxy->options(sql::mysql::MYSQL_PLUGIN_DIR, *p_s);
+    } else if (it != properties.end()) {
+      // Throw only when OPT_PLUGIN_DIR is used, but no value is given
+      throw sql::InvalidArgumentException("No string value passed for pluginDir");
     }
   }
 
@@ -695,66 +729,83 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
       } else {
         throw sql::InvalidArgumentException("No string value passed for characterSetResults");
       }
-    } else if (!it->first.compare(OPT_SSL_KEY)) {
+    } else if (!it->first.compare(OPT_SSL_KEY) || !it->first.compare("sslKey")) {
       try {
         p_s = (it->second).get< sql::SQLString >();
       } catch (sql::InvalidArgumentException&) {
-        throw sql::InvalidArgumentException("Wrong type passed for sslKey expected sql::SQLString");
+        throw sql::InvalidArgumentException("Wrong type passed for ssl-key expected sql::SQLString");
       }
       if (p_s) {
         sslKey = *p_s;
       } else {
-        throw sql::InvalidArgumentException("No string value passed for sslKey");
+        throw sql::InvalidArgumentException("No string value passed for ssl-key");
       }
       ssl_used = true;
-    } else if (!it->first.compare(OPT_SSL_CERT)) {
+    } else if (!it->first.compare(OPT_SSL_CERT) || !it->first.compare("sslCert")) {
       try {
         p_s = (it->second).get< sql::SQLString >();
       } catch (sql::InvalidArgumentException&) {
-        throw sql::InvalidArgumentException("Wrong type passed for sslCert expected sql::SQLString");
+        throw sql::InvalidArgumentException("Wrong type passed for ssl-cert expected sql::SQLString");
       }
       if (p_s) {
         sslCert = *p_s;
       } else {
-        throw sql::InvalidArgumentException("No string value passed for sslCert");
+        throw sql::InvalidArgumentException("No string value passed for ssl-cert");
       }
       ssl_used = true;
-    } else if (!it->first.compare(OPT_SSL_CA)) {
+    } else if (!it->first.compare(OPT_SSL_CA) || !it->first.compare("sslCA") ) {
       try {
         p_s = (it->second).get< sql::SQLString >();
       } catch (sql::InvalidArgumentException&) {
-        throw sql::InvalidArgumentException("Wrong type passed for sslCA expected sql::SQLString");
+        throw sql::InvalidArgumentException("Wrong type passed for ssl-ca expected sql::SQLString");
       }
       if (p_s) {
         sslCA = *p_s;
       } else {
-        throw sql::InvalidArgumentException("No string value passed for sslCA");
+        throw sql::InvalidArgumentException("No string value passed for ssl-ca");
       }
       ssl_used = true;
-    } else if (!it->first.compare(OPT_SSL_CAPATH)) {
+    } else if (!it->first.compare(OPT_SSL_CAPATH) || !it->first.compare("sslCAPath")) {
       try {
         p_s = (it->second).get< sql::SQLString >();
       } catch (sql::InvalidArgumentException&) {
-        throw sql::InvalidArgumentException("Wrong type passed for sslCAPath expected sql::SQLString");
+        throw sql::InvalidArgumentException("Wrong type passed for ssl-capath expected sql::SQLString");
       }
       if (p_s) {
         sslCAPath = *p_s;
       } else {
-        throw sql::InvalidArgumentException("No string value passed for sslCAPath");
+        throw sql::InvalidArgumentException("No string value passed for ssl-capath");
       }
       ssl_used = true;
-    } else if (!it->first.compare(OPT_SSL_CIPHER)) {
+    } else if (!it->first.compare(OPT_SSL_CIPHER) || !it->first.compare("sslCipher")) {
       try {
         p_s = (it->second).get< sql::SQLString >();
       } catch (sql::InvalidArgumentException&) {
-        throw sql::InvalidArgumentException("Wrong type passed for sslCipher expected sql::SQLString");
+        throw sql::InvalidArgumentException("Wrong type passed for ssl-cipher expected sql::SQLString");
       }
       if (p_s) {
         sslCipher = *p_s;
       } else {
-        throw sql::InvalidArgumentException("No string value passed for sslCipher");
+        throw sql::InvalidArgumentException("No string value passed for ssl-cipher");
       }
       ssl_used = true;
+    } else if (!it->first.compare(OPT_TLS_VERSION) || !it->first.compare("OPT_TLS_VERSION")) {
+      try {
+        p_s = (it->second).get< sql::SQLString >();
+      } catch (sql::InvalidArgumentException&) {
+        throw sql::InvalidArgumentException("Wrong type passed for OPT_TLS_VERSION expected sql::SQLString");
+      }
+      if (p_s) {
+        try {
+          proxy->options(sql::mysql::MYSQL_OPT_TLS_VERSION, *p_s);
+        }  catch (const sql::InvalidArgumentException&) {
+          //We will not throw error here, but wait for connection error
+          //libmysqlclient treats not valid TLS versions as invalid options.
+        }
+
+      } else {
+        throw sql::InvalidArgumentException("No string value passed for OPT_TLS_VERSION");
+      }
     } else if (!it->first.compare(OPT_DEFAULT_STMT_RESULT_TYPE)) {
       try {
         p_i = (it->second).get< int >();
@@ -1047,6 +1098,10 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 
   it = properties.find(OPT_SSL_MODE);
 
+  //Use legacy option
+  if(it == properties.end())
+    it = properties.find("OPT_SSL_MODE");
+
   if (it != properties.end())
   {
      PROCESS_CONN_OPTION(int, intOptions);
@@ -1103,7 +1158,13 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
                        " the password with mysql client that is capable to do that,"
                        " or rebuild your instance of Connector/C++ against mysql client"
                        " library that supports resetting of an expired password.";
-      } else {
+      } else if(native_error == CR_SSL_CONNECTION_ERROR){
+        error_message= proxy->error();
+        if(error_message.find("TLS version") != std::string::npos)
+        {
+          error_message+=", valid versions are: TLSv1.2, TLSv1.3";
+        }
+      }else {
         error_message= proxy->error();
       }
 
@@ -1145,6 +1206,64 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 
   }
 
+  /*
+   * Helper class to simplify setting and resetting of the plugin callback.
+   */
+  struct Fido_Callback_Setter
+  {
+    NativeAPI::NativeConnectionWrapper *proxy = nullptr;
+
+    /*
+     * Construct the setter using the callback and proxy.
+     */
+    Fido_Callback_Setter(Fido_Callback *callback,
+      NativeAPI::NativeConnectionWrapper *_proxy) :
+        proxy(_proxy)
+    {
+      if (proxy && callback && *callback)
+      {
+        callback_mutex.lock();
+        try
+        {
+          fido_callback_instance = callback;
+          proxy->plugin_option(MYSQL_CLIENT_AUTHENTICATION_PLUGIN,
+            "authentication_fido_client",
+            "fido_messages_callback",
+            (const void*)fido_callback_func
+          );
+
+        }
+        catch (sql::InvalidArgumentException& e) {
+          throw ::sql::SQLUnsupportedOptionException(
+            "Failed to set fido message callback for authentication_fido_client plugin",
+            OPT_OCI_CONFIG_FILE
+          );
+        }
+      }
+    }
+
+    ~Fido_Callback_Setter()
+    {
+      if(fido_callback_instance && proxy)
+      {
+        try
+        {
+          proxy->plugin_option(MYSQL_CLIENT_AUTHENTICATION_PLUGIN,
+            "authentication_fido_client",
+            "fido_messages_callback",
+            nullptr);
+        }
+        catch(...) {}
+        fido_callback_instance = nullptr;
+        callback_mutex.unlock();
+      }
+    }
+  };
+
+  Fido_Callback_Setter setter(
+    static_cast<MySQL_Driver*>(driver)->fido_callback,
+    proxy.get());
+
   //Connect loop
   {
     bool connected = false;
